/*
** Main.c module for Chapter9 Example (Stage10)
**
** Author:  John Hyde, USB Design By Example
*/

// Include global definitions, constants, etc
#include "main.h"

rom char BlinkStarted[] = "\nBlink (ID=%d) started";
rom char SlaveOpenned[] = "\nSlave Openned, SETUP thread (%d) started";
rom char SetupReceived[] = "\nSetup received:";
rom char FindKeyboardsStarted[] = "\nFindKeyboards (%d) started ";
rom char KeyboardFound[] = "\n\nKeyboard found (%X)";
rom char FindUserKeyboardStarted[] = "\nFindUserKeyboard (%d) started ";
rom char UserKeyboardFound[] = "\n\nUserKeyboard found (%X)";
rom char SendReportsStarted[] = "\nSendReports (%d) Started ";
rom char MemChkIs[] = "\nMemChk(%d)=";
rom char TCBPtrIs[] = "\nTCB_Ptr(%d)=";
rom char PerCentDandX[] = "%D (0x%X)";
rom char StackAllocated[] = ", Stack Allocated = %D";
rom char StackUsed[] = ", Stack Used = %D";
rom char CountIs[] = ", Count = %L";
rom char TCBIs[] = "\nContents of TCB(%d) are:";
rom char StatsDone[] = "\n\nStats done\n";

// Declare global variables
VOS_HANDLE hDevice[NUMBER_OF_DEVICES];
vos_semaphore_t DevicesStarted;
vos_mutex_t dprintLock;
WORD Delay;
MessageType KeyboardMessage;
char KeyboardName[] = "Keyboard";
vos_semaphore_t EnumerationComplete;
vos_semaphore_t SendKeyboardReport;
MessageType UserKeyboardMessage;
vos_semaphore_t SendUserKeyboardReport;
char UserKeyboardName[] = "UserKeyboard";
vos_semaphore_t RealKeyboardFound;
usbhost_device_handle* Keyboards[2];
vos_tcb_t* TCB_Ptr[NUMBER_OF_THREADS+1];
void* MemChk[3];

void CheckStatus(BYTE Status, BYTE ErrorCode) {
    BYTE Reason;
    if (Status == 0) return;
// Set a breakpoint at the next line so that this error can be resolved
    Reason = ErrorCode;
    }

void* MemCheck(BYTE ID) {
// Used to determine the amount of free memory available
    MemChk[ID] = vos_malloc(16);
    vos_free(MemChk[ID]);
    return MemChk[ID];
    }

void DisplayStats(void) {
    BYTE i;
    WORD Temp;
    DWORD Count;
    for (i=0; i<(sizeof(MemChk)/sizeof(MemChk[0])); i++) {
        dprint(&MemChkIs[0], &i);
        dprint(&PerCentDandX[0], &MemChk[i]);
        }
    for (i=0; i<NUMBER_OF_THREADS; i++) {
        dprint(&TCBPtrIs[0], &i);
        dprint(&PerCentDandX[0], &TCB_Ptr[i]);
// How much stack has been allocated to this thread, and how much has been used?
        Temp = TCB_Ptr[i+1] - TCB_Ptr[i] - 56;
        dprint(&StackAllocated[0], &Temp);
        Temp = vos_stack_usage(TCB_Ptr[i]);
        dprint(&StackUsed[0], &Temp);
        Count = vos_get_profile(TCB_Ptr[i]);
        dprint(&CountIs[0], &Count);
        }
// Replace (0) with (1) to display contents of each TCB
#if (0)
    for (i=0; i<NUMBER_OF_THREADS; i++) {
        dprint(&TCBIs[0], &i);
        dprintBuffer((BYTE*)TCB_Ptr[i], (int)(TCB_Ptr[i+1] - TCB_Ptr[i]));
        vos_delay_msecs(4000); // Don't flood V2EvalTerm
        }
#endif
    dprint(&StatsDone[0], 0);
    }

void WaitForDevicesStarted(void) {
    vos_wait_semaphore(&DevicesStarted);
    vos_signal_semaphore(&DevicesStarted);
    }

// Declare program thread(s)
// Blink thread, toggles LED3 on GPIO1. Stop blinking during Recording
void Blink(BYTE ThreadID) {
    BYTE Value;
    WORD Time = 0;
    BYTE PortData = LED3;
    vos_start_profiler();
    WaitForDevicesStarted();
    dprint(&BlinkStarted[0], &ThreadID);
    while (1) {
        i_vos_delay_msecs(ThreadID, Delay);
        PortData ^= LED3;
        Value = PortData;
// Now write pattern to the GPIO port.
        i_vos_dev_write(ThreadID, hDevice[LEDs] ,&Value, 1, NULL);
// After about 1 minute display some stats
        if (++Time == 250) DisplayStats();
        }
    }

void EnumerateSlave (BYTE ThreadID) {
    usbslave_ioctl_cb_t iocb;
    BYTE SetupBuffer[9];
    BYTE Status;
    WaitForDevicesStarted();
    dprint(&SlaveOpenned[0], &ThreadID);
    iocb.ioctl_code = VOS_IOCTL_USBSLAVE_WAIT_SETUP_RCVD;
    iocb.request.setup_or_bulk_transfer.buffer = &SetupBuffer[0];
    iocb.request.setup_or_bulk_transfer.size = sizeof(SetupBuffer);
    while (1) {
        Status = i_vos_dev_ioctl(ThreadID, hDevice[Slave], &iocb);
        dprint(&SetupReceived[0], 0);
        dprintBuffer(&SetupBuffer[0], sizeof(SetupBuffer));
        HandleRequest(ThreadID, (usb_deviceRequest_t*)&SetupBuffer[0]);
        }
    }

void FindKeyboards(BYTE ThreadID) {
// This thread looks for a keyboards attached to Host[2]
// It owns KeyboardMessage at startup
    usbhost_device_handle* ifDev;
    WaitForDevicesStarted();
    dprint(&FindKeyboardsStarted[0], &ThreadID);
    while (1) {
        ifDev = WaitForKeyboards(ThreadID);
        dprint(&KeyboardFound[0], &ifDev);
        vos_signal_semaphore(&RealKeyboardFound);
        KeyboardMessage.ReportLength = 8;
        KeyboardMessage.SignalSemaphore = &SendKeyboardReport;
        KeyboardMessage.NamePtr = &KeyboardName[0];
        GetReports(ThreadID, ifDev, &KeyboardMessage);
        }
    }

void FindUserKeyboard(BYTE ThreadID) {
// This thread waits for FindKeyboards to find 2 keyboards
// It owns UserKeyboardMessage at startup
    usbhost_device_handle* ifDev;
    WaitForDevicesStarted();
    dprint(&FindUserKeyboardStarted[0], &ThreadID);
    while (1) {
        vos_wait_semaphore(&RealKeyboardFound);
        ifDev = Keyboards[1];
        dprint(&UserKeyboardFound[0], &ifDev);
        UserKeyboardMessage.SignalSemaphore = &SendUserKeyboardReport;
        UserKeyboardMessage.ReportLength = 8;
        UserKeyboardMessage.NamePtr = &UserKeyboardName[0];
        GetReports(ThreadID, ifDev, &UserKeyboardMessage);
        }
    }

void SendReports(BYTE ThreadID) {
// Now checks two semaphores
    BYTE Status, Index;
    MessageType* Message;
    usbslave_ioctl_cb_t iocb;
    vos_semaphore_list_t* SemaphoreList;
    WaitForDevicesStarted();
    dprint(&SendReportsStarted[0], &ThreadID);
    SemaphoreList = (vos_semaphore_list_t*) vos_malloc(VOS_SEMAPHORE_LIST_SIZE(2));
    iocb.ioctl_code = VOS_IOCTL_USBSLAVE_TRANSFER;
    while (1) {
// Initialize the semaphore list that I will wait upon
        SemaphoreList->next = NULL;
        SemaphoreList->siz = 2;
        SemaphoreList->flags = VOS_SEMAPHORE_FLAGS_WAIT_ANY;
        SemaphoreList->list[0] = &SendKeyboardReport;
        SemaphoreList->list[1] = &SendUserKeyboardReport;
// Wait for ownership of KeyboardMessage (Index=0) or UserKeyboardMessage (Index=1)
        Index = vos_wait_semaphore_ex(SemaphoreList);
        if (Index) Message = &UserKeyboardMessage;
        else Message = &KeyboardMessage;
        iocb.handle = Message->DataEP;
        iocb.request.setup_or_bulk_transfer.buffer = Message->ReportBuffer;
        iocb.request.setup_or_bulk_transfer.size = Message->ReportLength;
        Status = i_vos_dev_ioctl(ThreadID, hDevice[Slave], &iocb);
        CheckStatus(Status, ErrorCouldNotSendReport);
// Give back Message
        i_vos_signal_semaphore(ThreadID, Message->ResponseSemaphore);
        }
    }

void MyIdleTask(void) {
// This thread is always ready to run and effectively replaces VOS's IdleTask
// I toggle bit 7 of the Logic Analyser Port to show activity
    StartupDevices();
    while (1) {
        asm { ORPORT 393 $0x80; };
        asm {ANDPORT 393 $0x7F; };
        }
    }


// Declare Main application
void main(void) {
// Initialize variables
    vos_init_semaphore(&DevicesStarted, 0);
    vos_init_mutex(&dprintLock, VOS_MUTEX_UNLOCKED);
    Delay = 511;
    vos_init_semaphore(&EnumerationComplete, 0);
    vos_init_semaphore(&SendKeyboardReport, 0);
    vos_init_semaphore(&SendUserKeyboardReport, 0);
    vos_init_semaphore(&RealKeyboardFound, 0);

// Initialise RTOS
    vos_init(VOS_QUANTUM, VOS_TICK_INTERVAL, NUMBER_OF_DEVICES);
    MemCheck(0);

// Sets the CPU frequency of the connected device.
    vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);

// Initialise devices
    if (InitDevices()) {
        MemCheck(1);

// Initialise threads - pass a ThreadID to each thread
// Instrument memory usage to display in DisplayStats
        TCB_Ptr[0] = vos_create_thread(20, SmallTCB, &Blink, 1, 1);
        TCB_Ptr[1] = vos_create_thread(27, StandardTCB, &EnumerateSlave, 1, 16);
        TCB_Ptr[2] = vos_create_thread(26, StandardTCB, &FindKeyboards, 1, 4);
        TCB_Ptr[3] = vos_create_thread(25, StandardTCB, &SendReports, 1, 8);
        TCB_Ptr[4] = vos_create_thread( 1, StandardTCB, &MyIdleTask, 0);
        TCB_Ptr[5] = vos_create_thread(26, StandardTCB, &FindUserKeyboard, 1, 2);
        TCB_Ptr[6] = (vos_tcb_t*) MemCheck(2);

// Start Scheduler to kick off the created thread(s)
        vos_start_scheduler();
        }

// It is an error to get here, use a breakpoint to catch this
    while (1) {
        CheckStatus(1, ErrorSchedulerDidNotStart);
        }
    }

